#noEnv
#include <debug>
#include <funcs>
#include <sqlite>
#include <objects>
#include *i buildid.ahk

setWorkingDir %a_scriptdir%
setBatchLines -1

CF_CLIPBOARD_VIEWER_IGNORE := create_clipboard_format_if_needed("Clipboard Viewer Ignore")
uncopyable_formats := [CF_CLIPBOARD_VIEWER_IGNORE, CF_HDROP]
copyable_formats := [CF_UNICODE]
active=1

;configuration file
iniRead hotkey, clipman.ini, config, hotkey, ^!C
iniRead suspendkey, clipman.ini, config, suspendkey, !^+c
iniRead removedups, clipman.ini, config, removedups, 1
iniRead sounds_active, clipman.ini, config, sounds_active, 1
iniRead, save_position, clipman.ini, config, save_position, 0
;localization start
lcid := dllCall("GetUserDefaultLCID", "uint")
f=lang\%lcid%.ini
iniRead, msg_select_item, %f%, messages, select_item, Select Item
iniRead, msg_options_menu, %f%, messages, options_menu, &Options
iniRead, msg_options, %f%, messages, options, Options
iniRead, msg_exit, %f%, messages, exit, E&xit
iniRead, msg_close, %f%, messages, close, Close
iniRead, msg_save, %f%, messages, save, &Save
iniRead, msg_Removedups, %f%, messages, removedups, &Remove duplicates
iniRead, msg_hotkey, %f%, messages, hotkey, Hotkey:
iniRead, msg_suspendkey, %f%, messages, suspendkey, Suspend key:
iniRead, msg_dberror_title, %f%, messages, dberror_title, Error Opening Database
iniRead, msg_dberror, %f%, messages, dberror, The database cannot be opened. Clipman will exit.`nPossible causes: sqlite3.dll is missing; clipman.db is corrupt; You are running from source with a 64-bit build of autohotkey and a 32-bit sqlite3.dll. Exception message: `%emessage`%
iniRead, msg_about_menu, %f%, messages, about_menu, &About Clipman...
iniRead, msg_about_title, %f%, messages, about_title, About Clipman
iniRead, msg_sounds_active, %f%, messages, sounds_active, &Sounds active
iniRead, msg_save_position, %f%, messages, save_position, Save list &position
iniRead, msg_vacuum_db, %f%, messages, vacuum_db, &Vacuum database
;localization end

;Try to open the database. IF it fails, warn the user and exit.
try {
db := new ClipDb("clipman.db")
} catch e {
emessage := e.message
transform, msg_dberror_transformed, deref, %msg_dberror%
msgbox 16,%msg_dberror_title%,%msg_dberror_transformed%
exitapp
}

hotkey, %hotkey%, showGui
hotkey, %suspendkey%, toggleSuspend
hotkey, ifWinActive, %msg_select_item% ahk_class AutoHotkeyGUI
hotkey, ~delete, delete
hotkey, ifWinActive

menu, tray, tip, Clipman
menu, tray, NoStandard
menu, tray, add, %msg_select_item%, showGUI
menu, tray, default, %msg_select_item%
menu, tray, add, %msg_about_menu%, about
menu, tray, add, %msg_vacuum_db%, vacuum
menu, tray, add, %msg_options_menu%, options
menu, tray, add, %msg_exit%, exit

;Is the gui showing?
showing=0
return

showGui:
if (showing)
return
gui, add, ListView, AltSubmit vClipList, item
gui, add, button, default gGuiClose, %msg_close%
contents := db.getListItems()
for i, element in contents {
lv_add("", element[2])
}
if (save_position && current_row > 0 && current_row <= contents.maxIndex()) {
index := current_row
} else {
index := contents.maxIndex()
}
lv_modify(index, "focus select")
showing=1
gui, show, ,%msg_select_item%
return


OnClipboardChange:
  critical
  if (!isobject(db)) {
    return
  }
  if (ignore_next_change) {
  ignore_next_change=0
  return
  }
  sleep, 100

  if (canCopyClipboardContents() == false) {
  if (sounds_active == 1) {
    soundPlay sounds\skip.wav
  }
  return
  }
  if (showing == 1)
  return
  cb := new ClipboardWrapper()
  if cb.open() = 0 {
  msgbox error opening clipboard
  return
  }
  c := new clip()
  for index, format in copyable_formats {
    format_contents := cb.get_clipboard_data(format, size)
    if (format_contents == 0) {
      throw Exception("Error getting format from clipboard")
    }
    c.add_format_from_clipboard(format, format_contents, size)
  }
  id := db.exists(c)
  if (removedups) && (id) {
    db.exec("update items set time=" . time() . " where id=" . id)
  } else {
    db.store(c)
  }
  c.free()
  cb.close()
  if (sounds_active == 1) {
    soundPlay sounds\copy.wav
  }
  return

canCopyClipboardContents() {
  global uncopyable_formats
  global copyable_formats
  global active
  if (active == 0) {
    return false
  }
  for index, format in uncopyable_formats {
    if (is_clipboard_format_available(format)) {
      return false
    }
  }
  available := false
  for index, format in copyable_formats {
    if (is_clipboard_format_available(format)) {
      return true
    }
  }
  return false
  }

GuiClose:
critical
if (LV_GetCount() == 0) {
showing=0
gui, destroy
return
}
ignore_next_change=1
current_row := lv_getNext(0, "focused")
id := contents[current_row][1]
clip := db.clip_from_db(id)
cb := new ClipboardWrapper()
cb.open()
cb.empty()
clip.to_clipboard(cb)
cb.close()
showing=0
gui, destroy
return

GuiEscape:
showing=0
gui, destroy
return

exit:
exitapp
return

delete:
if (LV_GetCount() == 0)
return
rows := lv_selected_rows()
if (!rows.maxIndex())
return
move_to_row := rows[rows.maxIndex()]+1
if (lv_getCount() == move_to_row-1) {
move_to_row := move_to_row-2
}
lv_modify(move_to_row, "focus select")
for i, row in rows {
;i-1 is the number of rows we deleted so far.
id := contents[row-(i-1)][1]
db.remove_clip(id)
contents.remove(row-(i-1))
lv_delete(row-(i-1))
}
return

options:
;Don't allow this dialog to open twice
if (showing)
return
showing=1
gui, add, Text, ,%msg_hotkey%
gui, add, hotkey, vNewHotkey, %hotkey%
gui, add, Text, ,%msg_suspendkey%
gui, add, hotkey, vNewSuspendkey, %suspendkey%
gui, add, checkbox, checked%removedups% vremovedups, %msg_removedups%
gui, add, checkbox, checked%sounds_active% vsounds_active, %msg_sounds_active%
gui, add, checkbox, checked%save_position% vsave_position, %msg_save_position%
gui, add, button, default gbuttonSave, %msg_save%
gui, show, ,%msg_options%
return

buttonSave:
gui, submit
gui, destroy
showing=0
if (newHotkey != "") {
hotkey, %hotkey%, ,off
hotkey, %newHotkey%, showGui, on
hotkey := newHotkey
iniWrite, %hotkey%, clipman.ini, config, hotkey
}
if (newSuspendkey != "") {
hotkey, %suspendkey%, ,off
hotkey, %newSuspendkey%, toggleSuspend, on
suspendkey := newSuspendkey
iniWrite, %suspendkey%, clipman.ini, config, suspendkey
}
iniWrite, %removedups%, clipman.ini, config, removedups
iniWrite, %sounds_active%, clipman.ini, config, sounds_active
iniWrite, %save_position%, clipman.ini, config, save_position
return

about:
msgbox 0,%msg_about_title%,
(
Clipman by Tyler Spivey <tspivey@pcdesk.net>
Build ID: %buildid%
LCID: %lcid%
)
return
toggleSuspend:
if (active == 0) {
active := 1
menu, tray, tip, Clipman
if (sounds_active == 1) {
soundPlay sounds\on.wav
}
else {
trayTip,, Clipman on
}
}
else {
active := 0
menu, tray, tip, Clipman: off
if (sounds_active == 1) {
soundPlay sounds\off.wav
}
else {
trayTip,, Clipman off
}
}
return

vacuum:
gui, 2:show, ,Vacuuming
db.exec("vacuum")
gui, 2:destroy
return
