#include <memory>

CF_HDROP := 15
CF_UNICODE := 13

default_clipboard_formats := {1: "CF_TEXT", 2: "CF_BITMAP", 3: "CF_METAFILEPICT", 4: "CF_SYLK", 5: "CF_DIF", 6: "CF_TIFF", 7: "CF_OEMTEXT", 8: "CF_DIB", 9: "CF_PALETTE", 10: "CF_PENDATA", 11: "CF_RIFF", 12: "CF_WAVE", 13: "CF_UNICODETEXT", 14: "CF_ENHMETAFILE", 15: "CF_HDROP", 16: "CF_LOCALE" }
default_clipboard_format_names := {}

for k, v in default_clipboard_formats
default_clipboard_format_names[v] := k

clipboard_format_to_name(format_identifier) {
  global default_clipboard_formats
  name := default_clipboard_formats[format_identifier]
  if (name == "") {
    name := get_clipboard_format_name(format_identifier)
  }
  return name
}

clipboard_name_to_format(name) {
  global default_clipboard_format_names
  format := default_clipboard_format_names[name]
  return format
}

create_clipboard_format_if_needed(format_name) {
  format := clipboard_name_to_format(format_name)
  if (format == "") {
    format := dllCall("RegisterClipboardFormat", "str", format_name, "uint") 
  }
  return format
}

get_clipboard_format_name(format_identifier) {
  ;Given a clipboard format identifier, returns its name
  name := ""
  varsetcapacity(name, 514)
  dllCall("GetClipboardFormatName", "uint", format_identifier, "ptr", &name, "int", 256)
  varsetcapacity(name, -1)
  return name
}

is_clipboard_format_available(format_identifier) {
  return DllCall("IsClipboardFormatAvailable", "Uint", format_identifier)
}


class ClipboardWrapper {

  __new() {
    this.opened := 0
  }

  open() {
    result := dllCall("OpenClipboard", "uint", A_ScriptHwnd)
    if (result != 0) {
      this.opened := 1
    }
    return result
  }

  close() {
    dllCall("CloseClipboard")
    this.opened := 0
  }

  availableFormats() {
    results := []
    format := dllCall("EnumClipboardFormats", "uint", 0, "uint")
    if (format == 0) {
      ;either 1) an error, or 2) empty clipboard. Check for the error
      last_error := ALastError
      if (last_error > 0) {
        throw Exception("formats: error " . last_error)
      }
    }
  while (format > 0) {
    results.insert(format)
    format := dllCall("EnumClipboardFormats", "uint", format, "uint")
  }
  return results
  }


  ;Returns: pointer to data, or 0
  ;Data must be freed with HeapFree.
  get_clipboard_data(format, byRef size) {
    handle := dllCall("GetClipboardData", "uint", format, "ptr")
    size := dllCall("GlobalSize", "ptr", handle)
    if (size == 0) {
      return 0
    }
    data := dllCall("GlobalLock", "uint", handle, "ptr")
    dest := memory_allocate(size)
    dllCall("RtlMoveMemory", "ptr", dest, "ptr", data, "uint", size)
    dllCall("GlobalUnlock", handle)
    return dest
  }

  empty() {
    dllCall("EmptyClipboard")
  }

set_clipboard_data(format, data_ptr, size) {
hMem := ptr2global(data_ptr, size)
rc := dllCall("SetClipboardData", "uint", format, "ptr", hMem)
if (rc == 0) {
msgbox SetClipboardData failed %a_lasterror%
}
}
}
