hModule := DllCall("LoadLibrary", "Str", "sqlite3.dll", "Ptr")
StrPutVar(string, ByRef var, encoding)
{
    ; Ensure capacity.
    VarSetCapacity( var, StrPut(string, encoding)
        ; StrPut returns char count, but VarSetCapacity needs bytes.
        * ((encoding="utf-16"||encoding="cp1200") ? 2 : 1) )
    ; Copy or convert the string.
    return StrPut(string, &var, encoding)
}

open_db(file) {
db := 0
rc := dllCall("sqlite3\sqlite3_open16", "wstr", file, "ptr*", db, "cdecl int")
if (errorLevel)
throw exception("error calling sqlite3_open, ErrorLevel: " errorLevel, -1)
errorLevel := rc
return db
}

sqlite_Get_error(db) {
err := dllCall("sqlite3\sqlite3_errmsg16", "ptr", db, "cdecl wstr")
return err
}

prepare(db, sql) {
handle := 0
rc := dllCall("sqlite3\sqlite3_prepare16_v2", "ptr", db, "wstr", sql, "int", -1, "ptr*", handle, "ptr", 0, "cdecl int")
if (errorLevel) {
rc := 0
return
}
errorLevel := rc
if (rc)
throw Exception("prepare error: " rc " string: " sqlite_Get_error(db))
return handle
}

sqlite3_step(statement) {
return dllCall("sqlite3\sqlite3_step", "ptr", statement, "cdecl int")
}

;-1 = transient
sqlite3_bind_text16(statement, param, value, size, callback=-1) {
return dllCall("sqlite3\sqlite3_bind_text16", "ptr", statement, "uint", param, "ptr", value, "uint", size, "ptr", callback, "cdecl int")
}

sqlite3_bind_blob(statement, param, value, size, callback=-1) {
return dllCall("sqlite3\sqlite3_bind_blob", "ptr", statement, "uint", param, "ptr", value, "uint", size, "ptr", callback, "cdecl int")
}

sqlite3_bind_int(statement, param, value) {
if (statement == 0)
throw exception("statement cannot be null")
return dllCall("sqlite3\sqlite3_bind_int", "ptr", statement, "int", param, "int", value, "cdecl int")
}

sqlite3_bind_int64(statement, param, value) {
dllCall("sqlite3\sqlite3_bind_int64", "ptr", statement, "uint", param, "int64", value, "cdecl int")
}

sqlite_exec(db, sql) {
handle := prepare(db, sql)
if (handle == 0)
throw exception("Error preparing: " . sqlite_Get_error(db))
result := sqlite3_exec_prepared(db, handle)
dllCall("sqlite3\sqlite3_finalize", "ptr", handle, "cdecl int")
return result
}

sqlite3_exec_prepared(db, handle) {
if (handle == 0)
throw exception("Handle is null")
loop {
rc := sqlite3_step(handle)
;all done
if (rc == 101)
break

if (rc != 100) {
throw exception("Unexpected result from exec: " . rc "str: " sqlite_get_error(db))
}
}
}

sqlite3_finalize(handle) {
return dllCall("sqlite3\sqlite3_finalize", "ptr", handle, "cdecl int")
}
sqlite3_last_insert_rowid(db) {
return dllCall("sqlite3\sqlite3_last_insert_rowid", "ptr", db, "cdecl int64")
}

sqlite3_reset(statement) {
rc := dllCall("sqlite3\sqlite3_reset", "ptr", statement, "cdecl int")
if (rc != 0 || errorlevel)
msgbox % "sqlite3_reset: rc=" rc " errorlevel " errorlevel
}

sqlite3_clear_bindings(statement) {
rc := dllCall("sqlite3\sqlite3_clear_bindings", "ptr", statement, "cdecl int")
return rc
}

sqlite3_column_text16(statement, col) {
text := dllCall("sqlite3\sqlite3_column_text16", "ptr", statement, "int", col, "cdecl ptr")
if (text == 0) {
errorlevel := 1
return
}
str := strget(text, "UTF-16")
return str
}
sqlite3_column_int64(statement, col) {
text := dllCall("sqlite3\sqlite3_column_int64", "ptr", statement, "int", col, "cdecl int64")
return text
}

sqlite3_column_blob(statement, col) {
text := dllCall("sqlite3\sqlite3_column_blob", "ptr", statement, "int", col, "cdecl ptr")
return text
}

sqlite3_column_bytes(statement, col) {
text := dllCall("sqlite3\sqlite3_column_bytes", "ptr", statement, "int", col, "cdecl int")
return text
}
